%manifold_1D_stable.m, by Daniel Lyons & Sara Wang (March 2014):
%Purpose: Computes 1D stable manifolds for the CCFM system of 3 ODEs over
%the range lambda<-0.6667 (1 1D stable manifold) or the range
%-0.6667<lambda<-0.5018 (2 1D stable manifolds)
%The user should set the desired lambda value as lambda_min on line 85
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%filepath (needs to be changed locally to folder containing "CCFM_manifolds")
mypath = '/Users/danielclyons/Dropbox/CCFM_large_folder';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(strcat(mypath,'/CCFM_manifolds/functions/'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialize parameters
global c lambda;
c = 3.0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%solution tolerance (f_solve)
sol_tol = 1.0e-8;
%set viewpoint for 3D plots
azimuth = 52; elevation = 10;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute invariant manifold via numerical integration
ti = 0.0; h = -0.25; tf = -25;%-25;%integration time data
time = ti:h:tf; %set time array
save_time_start_index = 3;%find(time==-2.0);%start saving at time(save_time_start_index)

%numerical perturbation options
ode_system = 'odeccfm';
ode_system_fsolve = 'fluxgate';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%obtain 6 fixed points with 2D stable manifold
% %fixed points for lambda = -0.43
fixed_points_array_0p43 = [

     0.769708796 0.999622906 -0.999316952; %saddle (2D stable)
     0.999622906 -0.999316952 0.769708796; %(construct 2D stable manifolds)
    -0.999316952 0.769708796 0.999622906;  %non-existent for lambda=-0.44
    
    -0.769708796 -0.999622906 0.999316952;
    -0.999622906 0.999316952 -0.769708796;
      0.999316952 -0.769708796 -0.999622906;
 
 
    -0.970586289 -0.627477624 -0.887742722; % saddle (2D stable)
    -0.627477624 -0.887742722 -0.970586289;
    -0.887742722 -0.970586289 -0.627477624;

     0.970586289 0.627477624 0.887742722;
    0.627477624 0.887742722 0.970586289;
    0.887742722 0.970586289 0.627477624;
     
    0.497328593 0.733515654 0.98003978; %saddle (2D unstable)
    0.98003978 0.497328593 0.733515654;
    0.733515654 0.98003978 0.497328593;

    -0.497328593 -0.733515654 -0.98003978;
    -0.98003978 -0.497328593 -0.733515654;
    -0.733515654 -0.98003978 -0.497328593;
    
    -0.9994542667 0.8564065259 0.99962304 %async sinks (3D stable)
    0.99962304 -0.9994542667 0.8564065259
    0.8564065259 0.99962304 -0.9994542667

    0.9994542667 -0.8564065259 -0.99962304
    -0.99962304 0.9994542667 -0.8564065259
    -0.8564065259 -0.99962304 0.9994542667
    
     
      0.9166123443 0.9166123443 0.9166123443 %sync sinks (3D stable)
      -0.9166123443 -0.9166123443 -0.9166123443

      0 0 0 %3D source
];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 1
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  80;%750;
lambda_min = -0.55;
lambda_max = -0.43;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = fixed_points_array_0p43;
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
close all;
%set lambda value
lambda = lambda_min;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%change depending on lambda value
if (lambda > -0.6667) && (lambda < -0.5018)
    fixed_points_array = final_fixed_point_array([25:27],:);
    num_saddle_points = 2;
elseif lambda <= -0.6667
    fixed_points_array = final_fixed_point_array(27,:);
    num_saddle_points = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%compute jacobian and stability info
for ii = 1:1%length(fixed_points_array)
    fpa_jac = CCFM_Jac(fixed_points_array(ii,:));
    
    %evals = eig(fpa_jac)
    [type,eig_val]  = stability_info(fpa_jac);
    
    %0: no information
    %1: (sink: 3D stable manifold (3 eigenvalues with negative real part)
    %2: (source: 3D unstable manifold (3 eigenvalues with positive real part)
    %3: (saddle: 2D stable manifold (2 eigenvalues with negative real part)
    %4: (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
    if type == 0
        verb_state = 'unknown';
    elseif type == 1
        verb_state = 'sink (3D stable)';
    elseif type == 2
        verb_state = 'source (3D unstable)';
    elseif type == 3
        verb_state = 'saddle (2D stable)';
    elseif type == 4
        verb_state = 'saddle (2D unstable)';
    end

    %print out stability info
    disp(['(',num2str(fixed_points_array(ii,1)),',',num2str(fixed_points_array(ii,2)),',',num2str(fixed_points_array(ii,3)),'): ',verb_state]);
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
my_face_alpha = 0.5;
x_save = []; y_save = []; z_save = [];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Loop through fixed points with 2D stable manifold
for ii=1:num_saddle_points

    clear x y z;


    fixed_points = fixed_points_array(ii,:);

    %[x,y,z] = manifold_compute(fixed_points,time,save_time_start_index,delta_x,step,ode_system);

    time_vec = 0:-0.005:-100;
    
    
    delta_x = 0.01; step = 0.005;
    ode_system = 'odeccfm';

        %compute 1D manifolds
        %stability info
        x0 = fixed_points;
        DF = CCFM_Jac(x0);
        [eigvec,D] = eig(DF);
        for ee=1:3
            eigval(ee) = D(ee,ee);
        end
        %obtain the 2 eigenvectors of 2D linear eigenspace
        eigvec_stable = eigvec(:,eigval<0);
        %normalize eigenvector length
        u_vec = eigvec_stable/norm(eigvec_stable);
        perturb = 0.001;
        %choose initial condition that is a small perturbation
        %integrate trajectories forwards
        IC_1 = x0 + perturb*u_vec';
        [~,s_tr] = ode23(@odeccfm,time_vec,IC_1,[]);

        x = s_tr(:,1); y = s_tr(:,2); z = s_tr(:,3);
        
        chop_val = 1.5;%2;%1
        r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);

        x(r) = [];
        y(r) = [];
        z(r) = [];
        
        
        figure(3); hold on;
        plot3(x,y,z,'r-','linewidth',2);
        
        
        %opposite eigenvector direction
        IC_2 = x0 - perturb*u_vec';
        [~,s_tr] = ode23(@odeccfm,time_vec,IC_2,[]);

        x = s_tr(:,1); y = s_tr(:,2); z = s_tr(:,3);
        
        r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);

        x(r) = [];
        y(r) = [];
        z(r) = [];
        
        
        
        
        figure(3); hold on;
        plot3(x,y,z,'r-','linewidth',2);

end
%End loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(3);
xlabel('x_1','fontsize',18);ylabel('x_2','fontsize',18);zlabel('x_3','fontsize',18);
title(['\lambda = ',num2str(lambda)],'fontsize',18);
set(gca,'fontsize',18);
hold on; grid on;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot sample trajectories
%obtain stable limit cycle
x0 = [0.8 -1.2 1.4];
[~,s_tr] = ode23(@odeccfm,[0 100],x0,[]);%trans integration

x0 = s_tr(end,:);
[~,s_tr] = ode23(@odeccfm,[0 100],x0,[]);

hold on;
plot3(s_tr(:,1),s_tr(:,2),s_tr(:,3),'k-','linewidth',4);
axis([-1.5 1.5 -1.5 1.5 -1.5 1.5]);
view([116 22]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fp_3D_stable_sync = final_fixed_point_array(25:26,:);
origin = final_fixed_point_array(27,:);

color_hash = {'k.','go','ko','bo','ro'};
%{stable, saddle (2D unstable), saddle (2D stable)}
mrkr_sz = 9;

figure(3);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%change depending on lambda value
if (lambda > -0.6667) && (lambda < -0.5018)
    hold on;
    plot3(fp_3D_stable_sync(:,1),fp_3D_stable_sync(:,2),fp_3D_stable_sync(:,3),color_hash{5},'markersize',mrkr_sz);
    hold on;
    plot3(origin(:,1),origin(:,2),origin(:,3),color_hash{3},'markersize',mrkr_sz);
elseif lambda <= -0.6667
    hold on;
    plot3(origin(:,1),origin(:,2),origin(:,3),color_hash{5},'markersize',mrkr_sz);
end


%-0.6667<lambda <-0.5018
% hold on;
% plot3(fp_3D_stable_sync(:,1),fp_3D_stable_sync(:,2),fp_3D_stable_sync(:,3),color_hash{5},'markersize',mrkr_sz);
% hold on;
% plot3(origin(:,1),origin(:,2),origin(:,3),color_hash{3},'markersize',mrkr_sz);

%lambda<-0.6667
%hold on;
%plot3(origin(:,1),origin(:,2),origin(:,3),color_hash{5},'markersize',mrkr_sz);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %draw arrow
hold on;
arrow_handle = arrow3(s_tr(105,:),s_tr(106,:),'k',2.5);

