%stablemanifold_two_basins_lambda_4348.m, by Daniel Lyons & Sara Wang (March 2014):
%Purpose: Compute the 2D stable manifolds and 1D stable manifolds of saddle
%points for lambda=0.4348
%Note: This code may be used over the range: 0.4346<lambda<0.4349
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%filepath (needs to be changed locally to folder containing "CCFM_manifolds")
mypath = '/Users/danielclyons/Dropbox/CCFM_large_folder';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(strcat(mypath,'/CCFM_manifolds/functions/'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialize parameters
global c lambda;
c = 3.0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%solution tolerance (f_solve)
sol_tol = 1.0e-8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%set viewpoint for 3D plots
azimuth = 52;
elevation = 10;

%color_hash for plotting
color_hash = {'k.','go','ko'};
%{stable, saddle (2D unstable), saddle (2D stable)}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute invariant manifold via numerical integration
ti = 0.0; h = -0.25; tf = -25;%-25;%integration time data
time = ti:h:tf; %set time array
save_time_start_index = 3;%find(time==-2.0);%start saving at time(save_time_start_index)
%input function names
ode_system = 'odeccfm';
ode_system_fsolve = 'fluxgate';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% known fixed points (equilibria) for lambda = -0.43
fixed_points_array_0p43 = [

    0.497328593 0.733515654 0.98003978; %saddle (2D unstable)
    0.98003978 0.497328593 0.733515654;
    0.733515654 0.98003978 0.497328593;

    -0.497328593 -0.733515654 -0.98003978;
    -0.98003978 -0.497328593 -0.733515654;
    -0.733515654 -0.98003978 -0.497328593;


     0.769708796 0.999622906 -0.999316952; %saddle (2D stable)
     0.999622906 -0.999316952 0.769708796; %(construct 2D stable manifolds)
    -0.999316952 0.769708796 0.999622906;  %non-existent for lambda=-0.44
    
    -0.769708796 -0.999622906 0.999316952;
    -0.999622906 0.999316952 -0.769708796;
      0.999316952 -0.769708796 -0.999622906;
 
 
    -0.970586289 -0.627477624 -0.887742722; % saddle (2D stable)
    -0.627477624 -0.887742722 -0.970586289; %non-existent for lambda=0.4348
    -0.887742722 -0.970586289 -0.627477624;

     0.970586289 0.627477624 0.887742722;
    0.627477624 0.887742722 0.970586289;
    0.887742722 0.970586289 0.627477624;
     
    
    -0.9994542667 0.8564065259 0.99962304 %async sinks (3D stable)
    0.99962304 -0.9994542667 0.8564065259
    0.8564065259 0.99962304 -0.9994542667

    0.9994542667 -0.8564065259 -0.99962304
    -0.99962304 0.9994542667 -0.8564065259
    -0.8564065259 -0.99962304 0.9994542667
    
     
      0.9166123443 0.9166123443 0.9166123443 %sync sinks (3D stable)
      -0.9166123443 -0.9166123443 -0.9166123443

      0 0 0 %3D source
];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Do equilibrium continuation to obtain equilibria at lambda=0.4348
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 1
%initialize lambda array info
dir_cont = 1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  25;%173;%750;
lambda_min = -0.43;
lambda_max = 0.43;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = fixed_points_array_0p43;
%equilibrium continuation
delta_lambda = (lambda_max-lambda_min)/(num_lambda-1);
lambda_array = lambda_min:delta_lambda:lambda_max;

for ii=1:num_lambda
    lambda = lambda_array(ii);
    disp(['ii = ',num2str(ii),' out of ',num2str(num_lambda),', lambda = ',num2str(lambda)]);
    fixed_points_array = compute_fixed_points(ode_system_fsolve,fixed_points_array,sol_tol);
    
end
final_fixed_point_array = fixed_points_array;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 2
%initialize lambda array info
dir_cont = 1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  20;%1700;%750;
lambda_min = 0.43;
lambda_max = 0.4348;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
%fixed_points_array = final_fixed_point_array;
%equilibrium continuation
delta_lambda = (lambda_max-lambda_min)/(num_lambda-1);
lambda_array = lambda_min:delta_lambda:lambda_max;

for ii=1:num_lambda
    lambda = lambda_array(ii);
    disp(['ii = ',num2str(ii),' out of ',num2str(num_lambda),', lambda = ',num2str(lambda)]);
    final_fixed_point_array = compute_fixed_points(ode_system_fsolve,final_fixed_point_array,sol_tol);
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute jacobian and stability info at final lambda (lambda_max)
for ii = 1:length(final_fixed_point_array)
    fpa_jac = CCFM_Jac(final_fixed_point_array(ii,:));
    
    %evals = eig(fpa_jac)
    [type,eig_val]  = stability_info(fpa_jac);
    
    %0: no information
    %1: (sink: 3D stable manifold (3 eigenvalues with negative real part)
    %2: (source: 3D unstable manifold (3 eigenvalues with positive real part)
    %3: (saddle: 2D stable manifold (2 eigenvalues with negative real part)
    %4: (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
    if type == 0
        verb_state = 'unknown';
    elseif type == 1
        verb_state = 'sink (3D stable)';
    elseif type == 2
        verb_state = 'source (3D unstable)';
    elseif type == 3
        verb_state = 'saddle (2D stable)';
    elseif type == 4
        verb_state = 'saddle (2D unstable)';
    end

    %print out stability info
    disp(['(',num2str(final_fixed_point_array(ii,1)),',',num2str(final_fixed_point_array(ii,2)),',',num2str(final_fixed_point_array(ii,3)),'): ',verb_state]);
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all;
%set lambda value
lambda = lambda_max;

%save 
fixed_points_array = final_fixed_point_array([1:12 25:27],:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%circle parameters (2D stable manifold)
points = 201;
radius = 0.1;%3e-2;

my_face_alpha = 0.5;%transparency
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x_save = []; y_save = []; z_save = [];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
int_time_1D_manifolds = 0:-0.005:-15;
time_1D = length(int_time_1D_manifolds);
one_dim_manifolds_array_x = zeros(2*time_1D,6);
one_dim_manifolds_array_y = zeros(2*time_1D,6);
one_dim_manifolds_array_z = zeros(2*time_1D,6);
%Loop through fixed points with 2D stable manifold
for ii=1:12%size(fixed_points_array,1)

    disp([' ']);
    disp(['Computing Manifold ',num2str(ii)]);
    clear x y z;

    fixed_points = fixed_points_array(ii,:);

    %inert numerical parameters
    delta_x = 0.01;%0.001;%0.1;%(unused)
    step = 0.005;%0.001*(2/3);%0.0005;%0.001;%0.001;%0.05;%0.01;
    

    if ii > 6 && ii <= 12 %ii <=6
    %compute eigenvalues for 2D stable manifolds with integrating backwards
    %in time beginning with a circle of initial conditions with radius "r"
    %and "points" points in the 2D space spanned by the two eigenvectors
    %at the equilibrium "fixed_points" with exactly 2 eigenvalues with
    %negative real part
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %compute Jacobian, eigenvalues, eigenvector at saddle points with 2D stable
        %manifolds
        x0 = fixed_points;
        DF = CCFM_Jac(x0);
        [eigvec,D] = eig(DF);
        for ee=1:3
            eigval(ee) = D(ee,ee);
        end
        %obtain the 2 eigenvectors of 2D linear eigenspace
        eigvec_stable = eigvec(:,eigval<0);

        %obtain coordinates of circle with radius r in (x1,x2) plane
        [x1,x2] = planar_circle(points,radius);

        %perform linear mapping of unit circle onto plane in R3 spanned by W1,W2
        [x1_new,x2_new,x3_new] = curve_map_R2_R3(x1,x2,eigvec_stable(:,1),eigvec_stable(:,2),points);
        %translate mapping to saddle point
        x1_new = x1_new + x0(1);
        x2_new = x2_new + x0(2);
        x3_new = x3_new + x0(3);
        x_new = [x1_new' x2_new' x3_new'];
        remove_small_norm_points = 0;
        [x,y,z] = manifold_compute_circle(fixed_points,time,save_time_start_index,delta_x,step,ode_system,x_new,remove_small_norm_points);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %add the 2 1D stable manifolds with closest saddle points
        
            saddle_point_dist = zeros(6,1);

            for kk = 1:6
                saddle_point_dist(kk) = norm(fixed_points-fixed_points_array(kk,:));
            end
            %obtain indices of closest saddle points
            [~,manifold_1_index] = min(saddle_point_dist);
            saddle_point_dist(manifold_1_index) = saddle_point_dist(manifold_1_index) + max(saddle_point_dist);
            [~,manifold_2_index] = min(saddle_point_dist);
            
            x = [x one_dim_manifolds_array_x(:,manifold_1_index)' one_dim_manifolds_array_x(:,manifold_2_index)'];
            y = [y one_dim_manifolds_array_y(:,manifold_1_index)' one_dim_manifolds_array_y(:,manifold_2_index)'];
            z = [z one_dim_manifolds_array_z(:,manifold_1_index)' one_dim_manifolds_array_z(:,manifold_2_index)'];

            %remove excess points
            chop_val = 1.7;%2;%1
            r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);

            x(r) = [];
            y(r) = [];
            z(r) = [];

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % % filter out extra points within small tolerance of each other
        norm_tol = 0.01;%0.01;
        [x,y,z] = redundant_point_filter(x,y,z,norm_tol);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        FP =  final_fixed_point_array(25:26,:);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %plot invariant manifold points only
         figure(2); hold on;
         point_plot(x,y,z);grid on;
         scale = 1.5; axis([-scale scale -scale scale -scale scale]);
         view([116 22]);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %plot surfaces with Delaunay2_5D program
         figure(3); hold on;
         Delaunay_special_plot(x,y,z,my_face_alpha);
         scale = 1.5; axis([-scale scale -scale scale -scale scale]);
         view([116 22]); grid on;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %obtain 1D stable manifolds of 6 saddle points
    elseif ii <= 6 %ii>6 && ii<=12
        
        %reset arrays
        if ii==10
            x_save = []; y_save = []; z_save =[];
        end
        
        
        %stability info
        x0 = fixed_points;
        DF = CCFM_Jac(x0);
        [eigvec,D] = eig(DF);
        for ee=1:3
            eigval(ee) = D(ee,ee);
        end
        %obtain the 2 eigenvectors of 2D linear eigenspace
        eigvec_stable = eigvec(:,eigval<0);
        %normalize eigenvector length
        u_vec = eigvec_stable/norm(eigvec_stable);
        perturb = 0.001;
        %choose initial condition that is a small perturbation
        %integrate trajectories forwards
        IC_1 = x0 + perturb*u_vec';
        [~,s_tr] = ode23(@odeccfm,int_time_1D_manifolds,IC_1,[]);

        x = s_tr(:,1); y = s_tr(:,2); z = s_tr(:,3);
        
        %save 1D manifold information
        one_dim_manifolds_array_x(1:time_1D,ii) = x;
        one_dim_manifolds_array_y(1:time_1D,ii) = y;
        one_dim_manifolds_array_z(1:time_1D,ii) = z; 
        
        %remove points outside of [-chop_val chopval] range
        chop_val = 1.5;%2;%1
        r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);

        x(r) = [];
        y(r) = [];
        z(r) = [];
        
        
        %save points a small distance from the origin
        min_norm = sqrt(3)/4;
        r = [];
        for mm=1:length(x)
            if norm([x(mm) y(mm) z(mm)])<min_norm
                r = [r; mm];
            end
        end
    
        x_save = [x_save; x(r)];
        y_save = [y_save; y(r)];
        z_save = [z_save; z(r)];
        
        figure(3); hold on;
        plot3(x,y,z,'r-','linewidth',2);
        
        
        %opposite eigenvector direction
        IC_2 = x0 - perturb*u_vec';
        [~,s_tr] = ode23(@odeccfm,int_time_1D_manifolds,IC_2,[]);

        x = s_tr(:,1); y = s_tr(:,2); z = s_tr(:,3);
        
        %save 1D manifold information
        one_dim_manifolds_array_x(time_1D+1:2*time_1D,ii) = x;
        one_dim_manifolds_array_y(time_1D+1:2*time_1D,ii) = y;
        one_dim_manifolds_array_z(time_1D+1:2*time_1D,ii) = z;
        
       %remove points outside of [-chop_val chopval] range
        r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);

        x(r) = [];
        y(r) = [];
        z(r) = [];
        
        
        
        %save points a small distance from the origin
        r = [];
        for mm=1:length(x)
            if norm([x(mm) y(mm) z(mm)])<min_norm
                r = [r; mm];
            end
        end
    
        x_save = [x_save; x(r)];
        y_save = [y_save; y(r)];
        z_save = [z_save; z(r)];
        
        
        figure(3); hold on;
        plot3(x,y,z,'r-','linewidth',2);
        
        
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         %compute pyramid surface near origin with saved 1D stable manifold values
%         if ii==9
%              figure(3); hold on;
%              num_points_interp = 2;
%              [x_save,y_save,z_save] = linear_interp_3D(x_save,y_save,z_save,num_points_interp);
%              Delaunay_special_plot_double(x_save,y_save,z_save,my_face_alpha);
%              %plot3(x_save,y_save,z_save,'b*');
%              scale = 1.5; axis([-scale scale -scale scale -scale scale]);
%              view([116 22]); grid on;
%         end
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         
%         clear x y z;
%         
%         IC_2 = x0 - perturb*u_vec';
%         [~,s_tr] = ode23(@odeccfm,0:-0.1:-15,IC_2,[]);
%         %x = [x; s_tr(:,1)]; y = [y; s_tr(:,2)]; z = [z; s_tr(:,3)];
%         %x = x'; y = y'; z = z';
%         x = s_tr(:,1); y = s_tr(:,2); z = s_tr(:,3);
%         r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);
%         
%           
%         %save points a small distance from the origing
%         min_norm = 2*sqrt(3)/4;
%         r = [];
%         for mm=1:length(x)
%             if norm([x(mm) y(mm) z(mm)])<min_norm
%                 r = [r; mm];
%             end
%         end
%     
%         x_save = [x_save; x(r)];
%         y_save = [y_save; y(r)];
%         z_save = [z_save; z(r)];
%         
%         size(x)
%         size(x_save)
%         
%         figure(3); hold on;
%         plot3(x,y,z,'r-','linewidth',2);
        
    end

end
%End loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot sample trajectories
figure(3);hold on;
num_traj = 3; %number of trajectories
min = 0.8; max = 1.5;
delta = (max - min)/(num_traj - 1);
mygrid = min:delta:max; %mygrid = -1.0:0.5:1.0;
%FP_1 = [-1.000 -1.000 -1.000];
%FP_2 = -FP_1;
%fixed_points_array = zeros(2,3);
%fixed_points_array(1,:) = FP_1;
%fixed_points_array(2,:) = FP_2;
plot_trajectories(mygrid,FP,lambda);
axis([-1.5 1.5 -1.5 1.5 -1.5 1.5]);
view([65 6]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%save fixed points to arrays
%fp_2D_stable_1
fp_2D_stable_2 = final_fixed_point_array(7:12,:);
fp_2D_unstable = final_fixed_point_array(1:6,:);
%fp_3D_stable = 
fp_3D_stable_sync = final_fixed_point_array(25:26,:);
origin = final_fixed_point_array(27,:);


color_hash = {'k.','go','ko','bo','ro'};
%{stable, saddle (2D unstable), saddle (2D stable)}
mrkr_sz = 9;
for ii=3:3
    figure(ii);
      %hold on;
      %plot3(fp_2D_stable_1(:,1),fp_2D_stable_1(:,2),fp_2D_stable_1(:,3),color_hash{4},'markersize',mrkr_sz);
     hold on;
     plot3(fp_2D_unstable(:,1),fp_2D_unstable(:,2),fp_2D_unstable(:,3),color_hash{5},'markersize',mrkr_sz);
     hold on;
     plot3(fp_2D_stable_2(:,1),fp_2D_stable_2(:,2),fp_2D_stable_2(:,3),color_hash{4},'markersize',mrkr_sz);
    %hold on;
    %plot3(fp_3D_stable(:,1),fp_3D_stable(:,2),fp_3D_stable(:,3),color_hash{1},'markersize',2*mrkr_sz);
    hold on;
    plot3(fp_3D_stable_sync(:,1),fp_3D_stable_sync(:,2),fp_3D_stable_sync(:,3),color_hash{1},'markersize',2*mrkr_sz);
    hold on;
    plot3(origin(:,1),origin(:,2),origin(:,3),color_hash{3},'markersize',mrkr_sz);
end

