%unstable_hopf_2D_manifold.m, by Daniel Lyons & Sara Wang (March 2014):
%Purpose: Computes the 2D stable manifold of an unstable limit cycle
%Lambda range: -0.5018<lambda<-0.4475 and 0.4349<lambda<1.3333
%Note: An unstable limit cycle is imported from AUTO from XPPAUT
%   and corresponds to one of the 'ca*.txt' or 'c*.txt' files
%Lines 51-52 are set by the user
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%filepath (needs to be changed locally to folder containing "CCFM_manifolds")
mypath = '/Users/danielclyons/Dropbox/CCFM_large_folder';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(strcat(mypath,'/CCFM_manifolds/functions/'));
addpath(strcat(mypath,'/CCFM_manifolds/Hopf_lambda_minus_files/'));
addpath(strcat(mypath,'/CCFM_manifolds/Hopf_lambda_plus_files/'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialize parameters
global c lambda;
c = 3.0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Use this section for choosing lambda and...
%importing an unstable limit cycle
%range: 0.4349<lambda<1.3333

%File        | lambda
%c1.txt      | 1.3110
%c2.txt      | 1.2420
%c3.txt      | 1.1420
%c4.txt      | 1.0390
%curve*.dat  | 0.9335
%c6.txt      | 0.8283
%c7.txt      | 0.7559
%c8.txt      | 0.6464
%c9.txt      | 0.5415
%c10.txt     | 0.4592

%Use this section for importing an unstable limit cycle in the lambda
%range: -0.5018<lambda<-0.4475
%File | lambda
%ca1.dat | -0.50175
%ca2.dat | -0.4995
%ca3.dat | -0.4882
%ca4.dat | -0.4681
%ca5.dat | -0.4519
%ca6.dat | -0.4475

%choose lambda value and corresponding data file from tables above
%(Note: For lambda=0.9335, the code will run regardless of the datafile)
lambda = -0.4681;
datafile = 'ca4.dat';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%import datafiles
if lambda==0.9335
    %use for lambda=0.9335 only
    data1 = importdata('curve.dat');
    data2 = importdata('curve2.dat');
    data3 = importdata('curve3.dat');
    t = data1(:,1);
    x1 = data1(:,2);
    x2 = data2(:,2);
    x3 = data3(:,2);
elseif lambda>0
    %use for all lambda>0 except lambda=0.9335
    data = importdata(datafile);
    t = data.data(:,1);
    x1 = data.data(:,2);
    x2 = data.data(:,3);
    x3 = data.data(:,4);
    data.textdata(1)
elseif lambda<0
    data = importdata(datafile);
    t = data.data(:,1);
    x1 = data.data(:,2);
    x2 = data.data(:,3);
    x3 = data.data(:,4);
    data.textdata(2,1)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot unstable limit cycle
figure; plot3(x1,x2,x3,'b--','linewidth',2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%set viewpoint for 3D plots
azimuth = 116; elevation = 22;
%color_hash for plotting
color_hash = {'k.','go','ko'};
%{stable, saddle (2D unstable), saddle (2D stable)} 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%obtain fixed points
fixed_points = [0 0 0];
%fixed_points = compute_fixed_points();%(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute invariant manifold via numerical integration
ti = 0.0; h = -0.25; tf = -30;%,-10%integration time data, h = -0.25
time = ti:h:tf; %set time array
save_time_start_index = 2;%find(time==-2.0);%start saving at time(save_time_start_index)

%numerical perturbation options
delta_x = 0.001;
step = 0.001;

ode_system = 'odeccfm';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute unstable limit cycle from 'rectangle' of points
limit_cycle_array = [x1 x2 x3];
[x,y,z] = manifold_compute_ulc(fixed_points,time,save_time_start_index,delta_x,step,ode_system,limit_cycle_array);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % filter out extra points within small tolerance of each other
norm_tol = 0.01;
[x,y,z] = redundant_point_filter(x,y,z,norm_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot invariant manifold points only
figure;
point_plot(x,y,z);
 hold on; plot3(x1,x2,x3,'r--','linewidth',16);
 view([azimuth elevation]);
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot surfaces with Delaunay2_5D program
figure;
my_face_alpha = 0.25;

%use for -0.5018<lambda<-0.4475
if lambda < 0
    Delaunay_special_plot_double(x,y,z,my_face_alpha);
elseif lambda > 0
    %use for 0.4346<lambda<1.3333
    Delaunay_special_plot(x,y,z,my_face_alpha);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hold on;
%plot unstable limit cycle in 1st octant 
plot3(x1,x2,x3,'b--','linewidth',3);

%plot unstable limit cycle in 8th octant (-0.5018<lambda<-0.4475 only)
if lambda < 0
    plot3(-x1,-x2,-x3,'b--','linewidth',3);
end

%solve for synchronous equilibrium point near (1,1,1)
fp_single = [1,1,1];
fp_single = compute_fixed_points('fluxgate',fp_single,1e-8);
hold on; grid on;
plot3(fp_single(1),fp_single(2),fp_single(3),'k.','markersize',18);
plot3(-fp_single(1),-fp_single(2),-fp_single(3),'k.','markersize',18);
%add point at origin
plot3(0,0,0,'ko','markersize',9);
%change view point
view([65 6]);
axis([-1.5 1.5 -1.5 1.5 -1.5 1.5]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %draw line of points in to origin to complete limit cycle (optional)
% symm_line = zeros(1,47);
% symm_line(1,:) = (-0.23:0.01:0.23).*ones(1,47);
% symm_line(2,:) = symm_line(1,:);
% symm_line(3,:) = symm_line(1,:);
% hold on; h=plot3(symm_line(1,:),symm_line(2,:),symm_line(3,:),'g-');
% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %compute cone (optional)
% %cone parametrization in spherical coordinates (optional)
% phi = pi/600;%pi/1000;%pi/200;
% rho_array = 0.1:0.0005:0.8;%0.7 %0:0.0002:0.45; %0:0.0006:1.1;%0:0.0002:0.3;
% theta_array = 0:2*pi/5:2*pi;
% cone_T = cone_compute(phi,rho_array,theta_array);%compute cone
% %plot cone
% hold on;
% Delaunay_special_plot_double(cone_T(:,1),cone_T(:,2),cone_T(:,3),my_face_alpha);
% %Delaunay_special_plot(cone_T(:,1),cone_T(:,2),cone_T(:,3),my_face_alpha);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %plot sample trajectories
% num_traj = 3; %number of trajectories
% min = -1.5; max = 1.5;
% delta = (max - min)/(num_traj - 1);
% mygrid = min:delta:max; %mygrid = -1.0:0.5:1.0;
% 
% plot_trajectories(mygrid,FP,lambda);
% scale = 1.5;
% axis([-scale scale -scale scale -scale scale]);
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot sample trajectories
%obtain stable limit cycle where it exists (for lambda<-0.4346)
if lambda<0
    x0 = [0.8 -1.2 1.4];
    [~,s_tr] = ode23(@odeccfm,[0 100],x0,[]);%trans integration

    x0 = s_tr(end,:);
    [~,s_tr] = ode23(@odeccfm,[0 100],x0,[]);
    %compute distance between final point and each fixed point;
    %dist(1) = norm(s_tr(end,:)-FP_1);
    %dist(2) = norm(s_tr(end,:)-FP_2);

    %[dyn_attr_dist(ii), dyn_attr(ii)] = min(dist);
    hold on;
    plot3(s_tr(:,1),s_tr(:,2),s_tr(:,3),'k-','linewidth',4);
    axis([-1.5 1.5 -1.5 1.5 -1.5 1.5]);
    view([112 22]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ind = 31;
%hold on; arrow3([x1(ind) x2(ind) x3(ind)],[x1(ind+1) x2(ind+1) x3(ind+1)],'k',1);%2.5
% view([azimuth  elevation]);
% zoom([0.8]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


