%compute_fixed_points.m, by Daniel Lyons(March 2014)
%computes fixed points with f_solve given an ode system and initial
%solution guess

function FP = compute_fixed_points(ode_system,FP_guess,sol_tol)

global c lambda;

%number of fixed points
num_FP = size(FP_guess,1);

FP = zeros(num_FP,3);

%solve nonlinear system of equations for fixed points
opts = optimoptions('fsolve','display','off','tolfun',1.0e-10);
for ii=1:num_FP
    IC = FP_guess(ii,:);
    if sum(isnan(IC))==0
        [FP_sol,fval,exitflag] = fsolve(ode_system,IC,opts);
        %exitflag
        fval = max(fval);
    else
        FP_sol = NaN;
        exitflag = -57;
        fval = 1;
    end

    FP(ii,:) = FP_sol;
    %return error if solution of 0 was not reached
    if (exitflag ~= 1) || (abs(fval)>sol_tol)
        disp(['Error at lambda = ',num2str(lambda),':']);
        FP(ii,:) = NaN;
    end
    
end

