%manifold_compute_circle.m, by Daniel Lyons & Sara Wang (March 2014)
%Purpose: From a locus of initial points in the matrix x_new, numerical
%integration is done and saved at the nodes values specified in the array time (backward
%direction of numerical integration may be specified by the array time).

function [x,y,z] = manifold_compute_circle(fixed_points,time,save_time_start_index,delta_x,step,ode_system,x_new,remove_small_norm_points)

disp(['     Computing Manifold...']);

x = [fixed_points(1)];
y = [fixed_points(2)];
z = [fixed_points(3)];

for FP = 1:length(fixed_points(:,1))

     x0_1 = x_new(:,1);
     x0_2 = x_new(:,2);
     x0_3 = x_new(:,3);

    count = 1;
    
    % calculate final destination for each point
    count_total = length(x0_1) * length(x0_2) * length(x0_3);
    for i = 1:length(x0_1)

                    x0 = [x0_1(i) x0_2(i) x0_3(i)]; % Initial conditions
                    [t,s] = ode23(ode_system,time,x0,[]);
                    x = [x s(save_time_start_index:end,1)'];
                    y = [y s(save_time_start_index:end,2)'];
                    z = [z s(save_time_start_index:end,3)'];
                    
                    count = count + 1;

    end
    
    %remove solution points with abs(x_i)>1 for any i=1,2,or 3
    chop_val = 1.7;%2;%1
    r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);
    
    x(r) = [];
    y(r) = [];
    z(r) = [];
    
    
    if remove_small_norm_points
        %remove solution points that too close to the origin
        %(because these points form a line which prevents triangulation)
        min_norm = sqrt(3)/4;
        r = [];
        %r = find((abs(x) < chop_val) && (abs(y) < chop_val) && (abs(z) > chop_val));
        for ii=1:length(x)

            if norm([x(ii) y(ii) z(ii)])<min_norm
                r = [r; ii];
            end
        end

        x(r) = [];
        y(r) = [];
        z(r) = [];
    end
    
end


