%manifold_compute_ulc.m, by Daniel Lyons & Sara Wang (March 2014)
%From a 3 column matrix containing the limit cycle points imported from
%XPPAUT (limit_cycle_array), a "cube" of initial conditions, described by the
%numerical parameters delta_x and step, is specified as described in the
%appendix of the article. Numerical integration is done for each initial
%condition and solution values are saved at the temporal nodes described in
%the array time. The output points are saved to the arrays x,y,z.

function [x,y,z] = manifold_compute_ulc(fixed_points,time,save_time_start_index,delta_x,step,ode_system,limit_cycle_array)

disp(['     Computing Unstable Limit Cycle Manifold...']);

%downsample limit_cycle_array
ds_num = 1;%6
lc_array_ds = downsample(limit_cycle_array,ds_num);

my_points = length(lc_array_ds);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%choose points to integrate
x = [fixed_points(1)];
y = [fixed_points(2)];
z = [fixed_points(3)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%use other way with delta_x and step
%Loop over each point in unstable limit cycle

chop_val = 1.7;

%opts = odeset('abstol',1e-5,'reltol',1e-5);%integration tolerance

for nn = 1:my_points
    disp(['nn = ',num2str(nn),' out of ',num2str(my_points)]);
    disp([' ']);
    lc_point = lc_array_ds(nn,:);
    
    lc_point_dx = lc_point(1)-delta_x:step:lc_point(1)+delta_x;
    lc_point_dy = lc_point(2)-delta_x:step:lc_point(2)+delta_x;
    lc_point_dz = lc_point(3)-delta_x:step:lc_point(3)+delta_x;
    
    for ii = 1:length(lc_point_dx)
        for jj = 1:length(lc_point_dy)
            for kk = 1:length(lc_point_dz)
                 x0 = [lc_point_dx(ii) lc_point_dy(jj) lc_point_dz(kk)];
                 [~,s] = ode23(ode_system,time,x0,[]);
                                  
                 x = [x s(save_time_start_index:end,1)'];
                 y = [y s(save_time_start_index:end,2)'];
                 z = [z s(save_time_start_index:end,3)'];

            end
        end
    end
    
end
size(x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %remove solution points with abs(x_i)>1 for any i=1,2,or 3
    %chop_val = 1.5;
    r = find(x > chop_val | x < -chop_val | y > chop_val | y < -chop_val | z > chop_val | z < -chop_val);
    
    x(r) = [];
    y(r) = [];
    z(r) = [];
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

