%polydraw.m, by Daniel Lyons (March 2014)
%Given the 3 column matrix t (values specify indicies of boundary triangles
%stored in x,y,z), and boundary value cut_val, each boundary triangle is
%identified as a triangle with exactly 1 point satisfying: abs(x)<cut_val,
%abs(y)<cut_val, and abs(z)<cut_val, or exactly 2 points satisfying all of
%these inequalities. If 1 or 2 points satisy the above conditions, then
%the boundary triangles are replaced by 1 or 3 triangles, respectively. The
%indices of the new triangles are appended to the three-column matrix of
%inside triangles, tri_inside. If print_info is non_zero, then triangle
%statistics are printed to the command window.

function [tri_inside,tri_boundary_saved,x,y,z] = polydraw(x,y,z,t,tri_inside,cut_val)

%print triangle algorithm information to STDOUT (1) or not (0)
print_info = 0;

tri_boundary = t;
%input: tri_boundary is a "num_tri" by 3 matrix containing the indices of
%triangles formed from the x,y,z vectors.

%number of inside triangles
num_tri_inside = length(tri_inside);
%number of boundary triangles
num_tri_b  = length(tri_boundary);

inside_count = zeros(3,1);


tri_boundary_saved = [];%zeros(num_tri_b,3);

if print_info
    disp(['Number of triangles before boundary triangulation...']);
    disp(['Inside boundary (',num2str(num_tri_inside),')']);
    disp(['On boundary (',num2str(num_tri_b),')']);
    disp(['Computing boundary triangles...']);
end

num_new_triangles_method_1 = 0;
num_new_triangles_method_2 = 0;



for ii=1:num_tri_b
    
    
    %identify how many vertices triangle has "inside box"
    %(it must be 1 or 2 for the triangle to be a boundary triangle)
    
    %get triangle point indices
    t1 = t(ii,1); t2 = t(ii,2); t3 = t(ii,3);
    
    %get triangle points
    p1 = [x(t1) y(t1) z(t1)];    
    p2 = [x(t2) y(t2) z(t2)];    
    p3 = [x(t3) y(t3) z(t3)]; 
    
    %inside_count(ii) =  number of coordinates of point ii such that
    %abs(index)<cut_val
    inside_count(1) = length(find(abs(p1)<=cut_val));
    inside_count(2) = length(find(abs(p2)<=cut_val));
    inside_count(3) = length(find(abs(p3)<=cut_val));
    
    tri_points_inside = 0;
    %tri_points_boundary = 0;
    for jj=1:3
        ic = inside_count(jj);
        if ic == 3
            tri_points_inside = tri_points_inside + 1;
        end
    end
    
    
    %determine if triangle is inside boundary, on boundary or outside
    %boundary; save indices of relevant triangle
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%Method 1: triangle has one point inside boundary
    if tri_points_inside == 1
        
        %find point inside the boundary
        %inside_index = 1 (p1 inside), inside_index = 2 (p2 inside),
        %inside_index = 3 (p3 inside)
        inside_index = find(inside_count==3);
        %find indices of the 2 points outside the boundary
        outside_index = find(inside_count~=3);  
        
        boundary_pt_index = zeros(1,2);
        %find replacement point for 1st point oustide the boundary
        %by finding a point on the boundary
        %parametrization sigma(gamma) = inside_point + outside_point*gamma
        %with 0<gamma<1
        
        %loop throught both boundary points
        for kk = 1:2
            %find index and location of inside point
            inside_index_curr = t(ii,inside_index);
            inside_point = [x(inside_index_curr) y(inside_index_curr) z(inside_index_curr)];


            %find index and location of one outside point
            outside_index_curr = t(ii,outside_index(kk));
            outside_point = [x(outside_index_curr) y(outside_index_curr) z(outside_index_curr)];


            %compute slope between inside point and current outside point
            slope = outside_point - inside_point;

            %find coordinate of outside point that is outside boundary
            %(assumes that only 1 coordinate is outside boundary)
            out_coor_index = find(abs(outside_point)> cut_val);
            %out_coor_index = out_coor_index(1);
            if out_coor_index(1) == 1
                out_coor_val = x(outside_index_curr);
                in_coor_val = x(inside_index_curr);
            elseif out_coor_index(1) == 2
                out_coor_val = y(outside_index_curr);
                in_coor_val = y(inside_index_curr);
            elseif out_coor_index(1) == 3
                out_coor_val = z(outside_index_curr);
                in_coor_val = z(inside_index_curr);
            end
            
            %takes into account sign of outside point
            if out_coor_val >=0
                cut_val_sign = cut_val;
            elseif out_coor_val <0
                cut_val_sign = -cut_val;
            end
            %compute gamma value (with 0<gamma<1) s.t. interpolated point is on
            %boundary (i.e., = cut_val)
            gamma = (cut_val_sign - in_coor_val)/(out_coor_val-in_coor_val);

            %evaluate parametrization to find new point on boundary
            boundary_pt_comp = inside_point + slope*gamma;
            
            %ii
            %boundary_pt_comp
            
            x = [x boundary_pt_comp(1)];
            y = [y boundary_pt_comp(2)];
            z = [z boundary_pt_comp(3)];

            boundary_pt_index(kk) = num_tri_inside + ii;
        end
        
        %add a row of point indicies corresponding to the x,y,z values of
        %the new triangle to the triangles inside or on the boundary
        
        %1st point: inside point
        new_triangle_index = length(tri_inside)+1;
        tri_inside(new_triangle_index,1) = inside_index_curr;
        %2nd point: 1st outside point
        tri_inside(new_triangle_index,2) = length(x)-1;
        %3rd point: 2nd outside point
        tri_inside(new_triangle_index,3) = length(x);
        
        %save boundary triangle info
        tri_boundary_saved = [tri_boundary_saved; tri_inside(new_triangle_index,:)];
        
        %add 1 to number of triangles created with method 1
        num_new_triangles_method_1 = num_new_triangles_method_1 + 1;
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%Method 2: triangle has two points inside boundary
    elseif tri_points_inside == 2
            
        %find points inside the boundary
        %inside_index = 1 (p1 inside), inside_index = 2 (p2 inside),
        %inside_index = 3 (p3 inside)
        inside_index = find(inside_count==3);
        %find indices of the 2 points outside the boundary
        outside_index = find(inside_count~=3);  
        
        %we'll have 2 boundary points and 1 midpoint
        boundary_pt_index = zeros(1,2);
        
        %find index and location of inside point
        inside_index_curr = t(ii,inside_index);
        inside_index_curr_1 = inside_index_curr(1);
        inside_index_curr_2 = inside_index_curr(2);
        inside_point_1 = [x(inside_index_curr(1)) y(inside_index_curr(1)) z(inside_index_curr(1))];
        inside_point_2 = [x(inside_index_curr(2)) y(inside_index_curr(2)) z(inside_index_curr(2))];

        %find index and location of one outside point
        outside_index_curr = t(ii,outside_index);
        outside_point = [x(outside_index_curr) y(outside_index_curr) z(outside_index_curr)];
        
        
        %loop through both inside points
        for kk = 1:2
        
        if kk==1
            inside_point = inside_point_1;
            my_inside_index_curr = inside_index_curr_1;
        elseif kk==2
            inside_point = inside_point_2;
            my_inside_index_curr = inside_index_curr_2;            
        end
        
        %compute slope between inside point and current outside point
        slope = outside_point - inside_point;

        %find coordinate of outside point that is outside boundary
        %(assumes that only 1 coordinate is outside boundary)
        out_coor_index = find(abs(outside_point)> cut_val);
        %out_coor_index = out_coor_index(1);
        if out_coor_index(1) == 1
            out_coor_val = x(outside_index_curr);
            in_coor_val = x(my_inside_index_curr);
        elseif out_coor_index(1) == 2
            out_coor_val = y(outside_index_curr);
            in_coor_val = y(my_inside_index_curr);
        elseif out_coor_index(1) == 3
            out_coor_val = z(outside_index_curr);
            in_coor_val = z(my_inside_index_curr);
        end
        
        
        %takes into account sign of outside point
        if out_coor_val >=0
            cut_val_sign = cut_val;
        elseif out_coor_val <0
            cut_val_sign = -cut_val;
        end
        %compute gamma value (with 0<gamma<1) s.t. interpolated point is on
        %boundary (i.e., = cut_val)
        gamma = (cut_val_sign - in_coor_val)/(out_coor_val-in_coor_val);
        
        

        %evaluate parametrization to find new point on boundary
        boundary_pt_comp = inside_point + slope*gamma;

        %ii
        %boundary_pt_comp

        x = [x boundary_pt_comp(1)];
        y = [y boundary_pt_comp(2)];
        z = [z boundary_pt_comp(3)];

        %boundary_pt_index(kk) = num_tri_inside + ii;
        end
        
        %calculate midpoint of two inside points
        midpoint = 0.5*(inside_point_1 + inside_point_2);
        %add midpoint of two inside points to x,y,z arrays
        x = [x midpoint(1)];
        y = [y midpoint(2)];
        z = [z midpoint(3)];
        
        
        
        %save indices of 3 new triangles
        %(1): inside_point_1
        bound_pt_index_1 = length(x)-2;
        bound_pt_index_2 = length(x)-1;
        midpoint_index   = length(x);
        
        %1st triangle: inside_index_curr_1, bound_pt_index_1,midpoint_index
        new_triangle_index = length(tri_inside)+1;
        tri_inside(new_triangle_index,:) = [inside_index_curr_1 bound_pt_index_1 midpoint_index];
        %save boundary triangle info
        tri_boundary_saved = [tri_boundary_saved; tri_inside(new_triangle_index,:)];
        
        %2nd triangle: bound_pt_index_1,midpoint_index,bound_pt_index_2
        new_triangle_index = length(tri_inside)+1;
        tri_inside(new_triangle_index,:) = [bound_pt_index_1 midpoint_index bound_pt_index_2];
        %save boundary triangle info
        tri_boundary_saved = [tri_boundary_saved; tri_inside(new_triangle_index,:)];
        
        %3rd triangle: midpoint_index,bound_pt_index_2,inside_index_curr_2
        new_triangle_index = length(tri_inside)+1;
        tri_inside(new_triangle_index,:) = [midpoint_index bound_pt_index_2 inside_index_curr_2];
        %save boundary triangle info
        tri_boundary_saved = [tri_boundary_saved; tri_inside(new_triangle_index,:)];
        
        
        
%         %1st point: inside point
%         tri_inside(new_triangle_index,1) = inside_index_curr;
%         %2nd point: 1st outside point
%         tri_inside(new_triangle_index,2) = length(x)-1;
%         %3rd point: 2nd outside point
%         tri_inside(new_triangle_index,3) = length(x);
        
        num_new_triangles_method_2 = num_new_triangles_method_2 + 3;
    end
    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%remove triangles with edges that are too long where 1 of the vertices is
%near the origin (lambda=0.43)
max_edge_length=0.05; r = [];
dist_origin = 1e-1;
for ii= 1:length(tri_inside)

    %get triangle point indices
    t1 = tri_inside(ii,1); t2 = tri_inside(ii,2); t3 = tri_inside(ii,3);
    
    %get triangle points
    p1 = [x(t1) y(t1) z(t1)];    
    p2 = [x(t2) y(t2) z(t2)];    
    p3 = [x(t3) y(t3) z(t3)];
    
    length_edge_1 = norm(p1-p2);
    length_edge_2 = norm(p2-p3);
    length_edge_3 = norm(p3-p1);
    
    
    
    tri_length_test = length_edge_1>max_edge_length || length_edge_2>max_edge_length || length_edge_3>max_edge_length;
    point_norm_test = norm(p1)<dist_origin || norm(p2)<dist_origin || norm(p3)<dist_origin;
    
    if tri_length_test && point_norm_test
        r = [r; ii];
    end
end
tri_inside(r,:) = [];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if print_info
    disp([' ']);
    disp(['Done!']);
    disp(['Number of triangles after boundary triangulation...']);
    disp(['Method 1, 1 point inside (',num2str(num_new_triangles_method_1),')']);
    disp(['Method 2, 2 points inside (',num2str(num_new_triangles_method_2),')']);
    disp(['Total number of inside triangles(',num2str(length(tri_inside)),')']);

    %directly confirm that none of triangles to be plotted are outside boundary
    %box
    disp(['Final check:']);
    [~,~,~] = tri_calc(x,y,z,tri_inside,cut_val);
end



