%tri_calc.m, by Daniel Lyons (March 2014)
%Given the indicies of triangles stored in the three column array t
%corresponding to points in the arrays x,y,z, each triangle is classified
%as an inside triangle (tri_inside), a boundary triangle (tri_boundary), or
%an outside triangle (tri_outside), and is stored in the corresponding
%array. Triangles are categorized according to the number of vertices that
%satisfy the following condition:

%abs(x)<cut_val, abs(y)<cut_val, and abs(z)<cut_val

%Triangles are classified according to the following table:

%Number of inside vertices|triangle classificiation
%-------------------------|------------------------------------------
%0                        |outside triangle (stored in tri_outside)
%1,2                      |boundary triangle (stored in tri_boundary)
%3                        |inside triangle (stored in tri_outside)

%Triangle output statistics are printed to the command window


function [tri_inside,tri_boundary,tri_outside] = tri_calc(x,y,z,t,cut_val)


%determine number of triangles inside boundary
tri_inside = [];
tri_boundary = [];
tri_outside = [];

num_tri= length(t);%number of triangles in complete triangulation

inside_count = zeros(3,1);


num_tri_inside = 0;
num_tri_boundary = 0;
num_tri_outside = 0;

for ii = 1:num_tri
    
    %get triangle point indices
    t1 = t(ii,1); t2 = t(ii,2); t3 = t(ii,3);
    
    %get triangle points
    p1 = [x(t1) y(t1) z(t1)];    
    p2 = [x(t2) y(t2) z(t2)];    
    p3 = [x(t3) y(t3) z(t3)]; 
    
    %inside_count(ii) =  number of coordinates of point ii such that
    %abs(index)<cut_val
    inside_count(1) = length(find(abs(p1)<=cut_val));
    inside_count(2) = length(find(abs(p2)<=cut_val));
    inside_count(3) = length(find(abs(p3)<=cut_val));
    
    tri_points_inside = 0;
    %tri_points_boundary = 0;
    for jj=1:3
        ic = inside_count(jj);
        if ic == 3
            tri_points_inside = tri_points_inside + 1;
        end
    end
    
    
    %determine if triangle is inside boundary, on boundary or outside
    %boundary; save indices of relevant triangle
    
    %triangle is inside boundary
    if tri_points_inside == 3
        num_tri_inside = num_tri_inside + 1;
        tri_inside = [tri_inside; t(ii,:)];
    %triangle is on boundary
    elseif tri_points_inside == 1 || tri_points_inside == 2
        num_tri_boundary = num_tri_boundary + 1;
        tri_boundary = [tri_boundary; t(ii,:)];
    %triangle is outside boundary
    elseif tri_points_inside == 0
        num_tri_outside = num_tri_outside + 1;
        tri_outside = [tri_outside; t(ii,:)];
    end
    
    
end

disp([' ']);
disp(['Number of triangles...']);
disp(['Inside boundary (',num2str(num_tri_inside),')']);
disp(['On boundary (',num2str(num_tri_boundary),')']);
disp(['Outside boundary (',num2str(num_tri_outside),')']);
disp(['Total (',num2str(num_tri),')']);




